#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

// Viteza CPU-ului si Baud rate:
#define FOSC 14745600
#define BAUD 9600

//Calculul timpilor USART
#define MYUBRR (((((FOSC * 10) / (16L * BAUD)) + 5) / 10) - 1)



// Masti petru statusul ledurilor 
#define LED_GREEN 0x01
#define LED_RED 0x02
#define BUTTON 0x08

//Folosim portul B pentru status si intrari
#define LED_PORT PORTB
#define BUTTON_PORT PORTB



// Masti pt selectarea nivelului
#define LAYER1 0x80
#define LAYER2 0x40
#define LAYER3 0x20
#define LAYER4 0x10
#define LAYERS 0xf0 // Toate nivelele
#define LAYERS_R 0x0f // Inversul nivelelor
#define LAYER_PORT PORTD // Folosim Portul D pentru nivele


// Folosim Port A si Port C pt coloanele cubului.
#define GRID1 PORTC
#define GRID2 PORTA


void ioinit (void); // Initializarea I/O 
void bootmsg (void); // Clipirea becurilor pentru a indica boot sau reboot
void delay_ms (uint16_t x); // functia delay folosita de la un capat la altul al programului
void led_red(unsigned char state); // led on or off
void led_green(unsigned char state);
void launch_effect (int effect); // incarcarea efectului.

// *** Cube buffer ***
// Memorarea cubului se face ntr-o matrice 2D de tipul volatile unsigned char cube[4][4]. 
//Matricea va avea doua dimensiuni si anume axa Z si axa Y. 
//Fiecare byte va reprezenta o linie de leduri de-a lungul axei X, asfel voi folosi doar cei 4 biti inferiori. 
//Matricea e de tipul volatile pentru a putea fi referita din functia de ntreruperi.
volatile unsigned char cube[4][4];


// temporary buffer folosit inainte de a scrie in buffer-ul cubului pentru a putea fi modificat usor.
volatile unsigned char tmpcube[4][4];

// nivelul pe care intrupere in arata.
volatile unsigned char current_layer;


// functii geometrice
#include "draw.c"

// Animatii statice
#include "frames.c"

// efectele pe care le va afisa cubul
#include "effect.c"


int main (void)
{
	
	// Initializarea porturilor IO
	ioinit();
	
	// Indicate that the device has just booted.
	
	bootmsg();

	int x;
	int i;
	int z;

	// Set the layer to start drawing at
	// Selectarea nivelului de unde incepem animatiile
	current_layer = 0x00;
	
	// Activarea intreruperilor pentru a incepe sa scriem in buffer-ul cubului
	// cand intreruperile vor fi activate ISR(TIMER2_COMP_vect)
	// vom rula la intervale de timp
	sei();

	
	while (1)
	{
		for (i=0;i<13;i++)
		{
			launch_effect(i);
		}
		
	}

}

// cele 13 efecte
void launch_effect (int effect)
{
	switch (effect)
	{
		
		// aprinderea fiecarui nivel unu cate unu
		case 0:
			loadbar(1000);
			break;
			
		// aprindrea a cate unui led aleator
		case 1:
			boingboing(150,500,0x03,0x01);
			break;
		
		// Umplerea cubului
		// Golirea lui aleator
		case 2:
			fill(0x00);
			random_filler(100,1,500,1);
			random_filler(100,1,500,0);
			break;
		
		
		// trimiterea de voxeli aleator pe axa z
		case 3:
			sendvoxels_rand_z(150,500,2000);
			break;
			
		// Spirala
		case 4:
			effect_spiral(1,75,1000);
			break;
		
		case 5:
			// toggle
			boingboing(150,500,0x03,0x02);
			break;
		
		// efect de ploaie
		case 6:
			effect_rain(20,5000,3000,500);
			break;
		
		
		case 7:
			boingboing(150,500,0x03,0x03);
			break;
			
		// Spinning plane
		case 8:
			effect_spinning_plane(1,50,1000);
			break;
		
		case 9:
			random_2();
			break;
		

		case 10:
			random_filler2(200,1);
			delay_ms(2000);
			random_filler2(200,0);
			delay_ms(1000);
			break;
			

		case 11:
			flyplane("z",1,1000);
			delay_ms(2000);
			flyplane("y",1,1000);
			delay_ms(2000);
			flyplane("x",1,1000);
			delay_ms(2000);
			flyplane("z",0,1000);
			delay_ms(2000);
			flyplane("y",0,1000);
			delay_ms(2000);
			flyplane("x",0,1000);
			delay_ms(2000);
			break;
			

		case 12:
			blinky2();
			break;
	}
}



// Setare sau nu a ledului rosu
void led_red(unsigned char state)
{
	if (state == 0x00)
	{
		LED_PORT &= ~LED_RED;
	} else
	{
		LED_PORT |= LED_RED;
	}
}

// Setarea sau nu a ledului verde
void led_green(unsigned char state)
{
	if (state == 0x00)
	{
		LED_PORT &= ~LED_GREEN;
	} else
	{
		LED_PORT |= LED_GREEN;
	}
}



// rutina de intruperi
ISR(TIMER2_COMP_vect)
{

	// dezactivam toate nivelurile
	//LAYER_PORT &= LAYERS_R;
	//LAYER_PORT &= 0xFF;
	LAYER_PORT &=0x00;
	// setarea coloanelor care vor fi aprinse
	GRID1 = (0x0f & cube[current_layer][0]) | (0xf0 & (cube[current_layer][1] << 4));
	GRID2 = (0x0f & cube[current_layer][2]) | (0xf0 & (cube[current_layer][3] << 4));

	// selectarea nivelului
	LAYER_PORT |= (0x01 << (5 - current_layer));

	// daca layer-ul curent ajunge la 3 o luam de la capat
	if (current_layer++ == 3)
		current_layer = 0;
}

void ioinit (void)
{
	
	
	// Data Direction Registers
	DDRA = 0xff;	// Interiorul cubului
	DDRB = 0xf7;	// ISP si 0-1: led. 3: button
	DDRC = 0xff;	// Exteriorul cubului
	DDRD = 0xff;	// Selectarea nivelului
	
	// selectarea tuturor porturilor sa fie oprite
	PORTA = 0x00;
	PORTC = 0x00;
	PORTB = 0x08; // Enable pull up button.
	PORTD = 0x00;

	
	

	// Timer pentru intreruperi
	TCNT2 = 0x00;	
	TIMSK |= (1 << OCIE2); 
	
	
	OCR2 = 15; 			// interrupt la counter = 15
	TCCR2 = 0x05; 		// prescaler = 1024
	TCCR2 |= (1 << WGM01);	// Clear Timer on Compare Match (CTC) mode


	// Initializare RS232
	// USART Baud rate: 9600
	UBRRH = MYUBRR >> 8;
	UBRRL = MYUBRR;
	
	UCSRC  = 0b10000110;
	//  RS232, tx si rx
	UCSRB = (1<<RXEN)|(1<<TXEN);
	UDR = 0x00; // byte pentru a indica powerup.

}



void bootmsg (void)
{
	int i;
	LED_PORT |= LED_GREEN;
	for (i = 0; i < 2; i++)
	{
		// Blinky
		delay_ms(1000);
		LED_PORT &= ~LED_GREEN;
		LED_PORT |= LED_RED;
		// Blink
		delay_ms(1000);
		LED_PORT &= ~LED_RED;
		LED_PORT |= LED_GREEN;
	}
	delay_ms(1000);
	LED_PORT &= ~LED_GREEN;
}

// delay-ul folosit pentru efecte.
void delay_ms(uint16_t x)
{
  uint8_t y, z;
  for ( ; x > 0 ; x--){
    for ( y = 0 ; y < 90 ; y++){
      for ( z = 0 ; z < 6 ; z++){
        asm volatile ("nop");
      }
    }
  }
}
